#include "WebFrontend.h"
#include <EEPROM.h>
#include "Settings.h"
#include "HTML.h"
#include "OTAUpdate.h"

const char on_log[] PROGMEM = ""
"<script>\n"
"  function sendCommand() {\n"
"    var cmd = document.getElementById('commandText').value;\n"
"    var request = new XMLHttpRequest();\n"
"    request.open('GET', 'command?cmd=' + encodeURIComponent(cmd), true);\n"
"    request.send();\n"
"  };\n"
"  function clearList(what) {\n"
"    document.getElementById(what + 'Div').innerHTML = '';\n"
"    filter(what);\n"
"  };\n"
"  function filter(what) {\n"
"    var el = document.getElementById(what + 'DivFilter');\n"
"    var text0 = el.value.toLowerCase();\n"
"    var elements = document.getElementsByClassName(what + 'Line');\n"
"    var names = '';\n"
"    var ct = 0;\n"
"    for (var i = 0; i < elements.length; i++) {\n"
"      if (elements[i].innerHTML.toLowerCase().indexOf(text0) == -1) {\n"
"        elements[i].style.display = 'none';\n"
"      }\n"
"      else {\n"
"        elements[i].style.display = 'block';\n"
"        ct++;\n"
"      }\n"
"    }\n"
"    document.getElementById(what + 'RowCount').innerHTML = ct + \" rows\";\n"
"  };\n"
"  function run() {\n"
"    var el = document.getElementById('logDivFilter');\n"
"    el.onkeyup = function (evt) {\n"
"      filter('log');\n"
"    };\n"
"    var el = document.getElementById('dataDivFilter');\n"
"    el.onkeyup = function (evt) {\n"
"      filter('data');\n"
"    };\n"
"    getLogData();\n"
"  };\n"
"  function getLogData() {\n"
"    if (document.getElementById('enabled').checked == true) {\n"
"      var request = new XMLHttpRequest();\n"
"      request.onreadystatechange = function () {\n"
"        if (this.readyState == 4 && this.status == 200 && this.responseText != null && this.responseText != '') {\n"
"          var lines = this.responseText.split('\\n');\n"
"          for (var i = 0; i < lines.length; i++) {\n"
"            var txt = lines[i];\n"
"            if (txt != '') {\n"
"              if (txt == 'SYS: ***CLEARLOG***') {\n"
"                clearList('data');\n"
"                clearList('log');\n"
"              } else {\n"
"                var targetDiv = 'logDiv';\n"
"                var scrollCheckBox = 'scrollLogDiv';\n"
"                var prefix = 'log';\n"
"                if (txt.startsWith('DATA:')) {\n"
"                  prefix = 'data';\n"
"                  targetDiv = 'dataDiv';\n"
"                  scrollCheckBox = 'scrollDataDiv';\n"
"                  txt = txt.substring(5);\n"
"                }\n"
"                if (txt.startsWith('SYS:')) {\n"
"                  txt = txt.substring(4);\n"
"                }\n"
"                txt = new Date().toLocaleTimeString('de-DE') + ': ' + txt;\n"
"                document.getElementById(targetDiv).innerHTML += \"<div class='\" + prefix + \"Line'>\" + txt + '</div>';\n"
"                filter(prefix);\n"
"                if (document.getElementById(scrollCheckBox).checked == true) {\n"
"                  var objDiv = document.getElementById(targetDiv);\n"
"                  objDiv.scrollTop = objDiv.scrollHeight;\n"
"                }\n"
"              }\n"
"            }\n"
"          }\n"
"        }\n"
"      };\n"
"      request.open('GET', 'getLogData?nc=' + Math.random(), true);\n"
"      request.send();\n"
"    }\n"
"    setTimeout('getLogData()', 500);\n"
"  };\n"
"</script>\n"
"<body onload='run()'>\n"
"  Command: <input id='commandText' size='100' onkeydown=\"if (event.keyCode == 13) sendCommand()\">\n"
"  <button type='button' onclick=\"sendCommand();\">Send</button>\n"
"  &nbsp;&nbsp;&nbsp;\n"
"  <input type='checkbox' id='enabled' value='true' checked>Enable logging\n"
"  <br><br>\n"
"  <i>LGW to FHEM:</i>\n"
"  <input type='checkbox' id='scrollDataDiv' value='true' checked> Scroll\n"
"  <button type='button' onclick=\"clearList('data');\">Clear</button>\n"
"  Filter:\n"
"  <input id='dataDivFilter'>\n"
"  <span id='dataRowCount'></span>\n"
"  <div id='dataDiv' style='height: 250px; border:1px solid black; overflow:scroll;'></div>\n"
"  <br><i>Debug log:</i>\n"
"  <input type='checkbox' id='scrollLogDiv' value='true' checked> Scroll\n"
"  <button type='button' onclick=\"clearList('log');\">Clear</button>\n"
"  Filter:\n"
"  <input text='text' id='logDivFilter'>\n"
"  <span id='logRowCount'></span>\n"
"  <div id='logDiv' style='height: 250px; border:1px solid black; overflow:scroll;'></div>\n"
"</body>\n"
;

WebFrontend::WebFrontend(int port) : m_webserver(port) {
  m_port = port;
}

ESP8266WebServer *WebFrontend::WebServer() {
  return &m_webserver;
}

void WebFrontend::SetCommandCallback(CommandCallbackType callback) {
  m_CommandCallback = callback;
}

void WebFrontend::Begin(StateManager *stateManager, Logger *logger) {
  m_stateManager = stateManager;
  m_logger = logger;

  m_webserver.on("/", [this]() {
    String result;
    result += GetTop();
    result += GetNavigation();
    result += m_stateManager->GetHTML();
    result += GetBottom();
    m_webserver.send(200, "text/html", result);
  });

  m_webserver.on("/reset", [this]() {
    m_webserver.send(200, "text/html", GetRedirectToRoot());
    ESP.restart();
  });

  m_webserver.on("/command", [this]() {
    if (m_CommandCallback != NULL) {
      String command = m_webserver.arg("cmd");
      m_logger->println("Command from frontend: '" + command + "'");
      m_CommandCallback(command);
      m_webserver.send(200, "text/html", "OK");
    }
  });

  m_webserver.on("/state", [this]() {
    String result;
    result += m_stateManager->GetXML();
    m_webserver.send(200, "text/xml", result);
  });

  m_webserver.on("/ota", [this]() {
    String result;
    result += GetTop();
    result += GetNavigation();
    Settings settings;
    settings.Read();
    result += F("<form method='get' action='ota_start'>");
    result += F("Server:&nbsp");
    result += settings.Get("otaServer", "");
    result += F("<br>Port:&nbsp");
    result += settings.Get("otaPort", "");
    result += F("<br>url:&nbsp");
    result += settings.Get("otaURL", "");
    result += F("<br><br> <input type='submit' Value='Update and restart' > </form>");
    result += GetBottom();
    m_webserver.send(200, "text/html", result);
  });

  m_webserver.on("/ota_start", [this]() {
    m_webserver.send(200, "text/html", OTAUpdate::Start());
  });

  m_webserver.on("/save", [this]() {
    Settings settings;
    
    for (byte i = 0; i < m_webserver.args(); i++) {
      settings.Add(m_webserver.argName(i), m_webserver.arg(i));
    }
            
    ////settings.Dump();
    settings.Write();

    m_webserver.send(200, "text/html", GetRedirectToRoot("Settings saved"));
    
    ESP.restart();
  });

  m_webserver.on("/setup", [this]() {
    String result;
    result += GetTop();
    result += GetNavigation();

    Settings settings;
    settings.Read();
           
    result += F("<form method='get' action='save'> <table>");
          
    // ctSSID
    result += F("<tr> <td> <label>SSID: </label> </td> <td> <input name='ctSSID' size='120' maxlength='32' Value='");
    result += settings.Get("ctSSID", "");
    result += F("'> </td> </tr>");
           
    // ctPASS
    result += F("<tr> <td> <label>Password: </label> </td> <td> <input name='ctPASS' size='120' maxlength='63' Value='");
    result += settings.Get("ctPASS", "");
    result += F("'> </td> </tr>");

    result += F("<tr><td></td><td><br>If IP, mask or gateway is empty, DHCP will be used</td></tr>");

    // staticIP
    result += F("<tr> <td> <label>IP-Address: </label> </td> <td> <input name='staticIP' size='120' maxlength='15' Value='");
    result += settings.Get("staticIP", ""); 
    result += F("'> </td> </tr>");
            
    // staticMask
    result += F("<tr> <td> <label>Netmask: </label> </td> <td> <input name='staticMask' size='120' maxlength='15' Value='");
    result += settings.Get("staticMask", "");
    result += F("'> </td> </tr>");
            
    // staticGW
    result += F("<tr> <td> <label>Gateway: </label> </td> <td> <input name='staticGW' size='120' maxlength='15' Value='");
    result += settings.Get("staticGW", "");
    result += "'> </td> </tr>";
            
    // HostName
    result += F("<tr> <td> <label>Hostname: </label> </td> <td> <input name='HostName' size='120' maxlength='63' Value='");
    result += settings.Get("HostName", "LaCrosseGateway");
    result += F("'> </td> </tr>");

    // Data ports
    result += F("<tr> <td> <label>Data ports: </label> </td> <td>");
    result += F("<input name='DataPort1' maxlength='5' Value='");
    result += settings.Get("DataPort1", "81");
    result += F("'>&nbsp;");
    result += F("<input name='DataPort2' maxlength='5' Value='");
    result += settings.Get("DataPort2", "");
    result += F("'>&nbsp;");
    result += F("<input name='DataPort3' maxlength='5' Value='");
    result += settings.Get("DataPort3", "");
    result += F("'>&nbsp;");
    result += F("</td> </tr>");

    // KVInterval
    result += F("<tr><td></td><td><br>Use 'off' to disable KV-transmission</td></tr>");
    result += F("<tr> <td> <label>KV-Interval: </label> </td> <td> <input name='KVInterval' size='120' maxlength='3' Value='");
    result += settings.Get("KVInterval", "10");
    result += F("'> </td> </tr>");

    // KVIdentity
    result += F("<tr> <td> <label>KV-Identity: </label> </td> <td> <input name='KVIdentity' size='120' maxlength='20' Value='");
    result += settings.Get("KVIdentity", String(ESP.getChipId()));
    result += F("'> </td> </tr>");
            
    result += F("<tr><td></td><td><br>OTA update</td></tr>");

    // OTA-Server
    result += F("<tr> <td> <label>OTA-Server: </label> </td> <td> <input name='otaServer' size='120' maxlength='40' Value='");
    result += settings.Get("otaServer", "");
    result += F("'> </td> </tr>");

    // OTA-Port
    result += F("<tr> <td> <label>OTA-Port: </label> </td> <td> <input name='otaPort' size='120' maxlength='5' Value='");
    result += settings.Get("otaPort", "");
    result += "'> </td> </tr>";

    // OTA-url
    result += F("<tr> <td> <label>OTA-url: </label> </td> <td> <input name='otaURL' size='120' maxlength='80' Value='");
    result += settings.Get("otaURL", "");
    result += F("'> </td> </tr>");

    // PCA301
    result += F("<tr><td></td><td><br>PCA301 (normally no need to change it)</td></tr>");
    result += F("<tr> <td> <label>Plugs: </label> </td> <td> <input name='PCA301Plugs' size='120' maxlength='160' Value='");
    result += settings.Get("PCA301Plugs", "");
    result += F("'> </td> </tr>");

    // Flags
    result += F("<tr><td></td><td><br>Only for development</td></tr>");
    result += F("<tr> <td> <label>Flags: </label> </td> <td> <input name='Flags' size='120' maxlength='80' Value='");
    result += settings.Get("Flags", "");
    result += F("'> </td> </tr>");

    result += F("</table> <br> <input type='submit' Value='Save and restart' > </form>");

    result += GetBottom();
    m_webserver.send(200, "text/html", result);
  });
  
  m_webserver.on("/getLogData", [this]() {
    String data = "";
    if (m_logger->IsEnabled()) {
      while (m_logger->Available()) {
        data += m_logger->Pop() + "\n";
      }
    }
    else{
      data += F("SYS: ***CLEARLOG***\n");
      data += F("DATA:Logger is disabled\n");
      data += F("SYS:Logger is disabled\n");
    }

    m_webserver.send(200, "text/html", data);
  });

  m_webserver.on("/log", [this]() {
    String result;
    result += GetTop();
    result += GetNavigation();

    result += FPSTR(on_log);
   
    result += GetBottom();

    m_webserver.send(200, "text/html", result);
  });


  m_webserver.onNotFound([this](){
    m_webserver.send(404, "text/plain", "Not Found");
  });

  m_webserver.begin();
}

String WebFrontend::GetNavigation() {
  String result = "";
  result += F("<a href='/'>Home</a>&nbsp;&nbsp;");
  result += F("<a href='setup'>Setup</a>&nbsp;&nbsp;");
  result += F("<a href='ota'>OTA-Update</a>&nbsp;&nbsp;");
  result += F("<a href='log'>Log</a>&nbsp;&nbsp;");
  result += F("<a href='reset'>Reboot</a>&nbsp;&nbsp;");
  result += F("<br><br>");
  
  return result;
}

String WebFrontend::GetTop() {
  String result;
  result += F("<!DOCTYPE HTML>\r\n<html>");
  result += F("<meta charset='utf-8'/>");
  result += F("<p>LaCrosseGateway V");
  result += m_stateManager->GetVersion();
  result += F("</p>");
  return result;
}

String WebFrontend::GetBottom() {
  String result;
  result += F("</html>");
  return result;
}

String WebFrontend::GetRedirectToRoot(String message) {
  String result;
  result += F("<html><head><meta http-equiv='refresh' content='5; URL=/'></head><body>");
  result += message;
  result += F("<br><br>Reboot, please wait a moment ...</body></html>");
  return result;
}


void WebFrontend::Handle() {
  m_webserver.handleClient();
}

